//
//  SYSegmentControl.m
//  blockCallbackReplaceDelegate
//
//  Created by 杨淳引 on 15/7/17.
//  Copyright (c) 2015年 杨淳引. All rights reserved.
//

#import "SYSegmentControl.h"

#define kDefauleSelectedIndex 0
#define kSelectedBottomLineWidth 2
#define kSelectedBottomLineColor [UIColor blueColor]
#define kButtonTextFont 13
#define kButtonDefaultTitleColor [UIColor blackColor]
#define kButtonDefaultBgColor [UIColor grayColor]
#define kButtonSelectedTitleColor [UIColor blueColor]
#define kButtonSelectedBgColor [UIColor whiteColor]

@interface SYSegmentControl ()

@property (nonatomic, strong) NSArray *itemsArray;//segmentControl的按钮标题数组
@property (nonatomic, strong) NSMutableArray *buttonsArray;//segmentControl的按钮数组
@property (nonatomic, assign) CGFloat buttonHeight;
@property (nonatomic, assign) CGFloat buttonWidth;
@property (nonatomic, strong) UIView *selectedBottomLine;//选中某个按钮时按钮下方的高亮线
@property (nonatomic, assign) NSInteger selectedIndex;//选中的按钮

@property (nonatomic, strong) callbackBlock block;//块回调模式使用的回调代码

@end

@implementation SYSegmentControl

#pragma mark - Lifecycle

- (instancetype)initWithTitles:(NSArray *)titles frame:(CGRect)frame delegate:(id<SYSegmentControlDelegate>) theDelegate {
    if (self = [super initWithFrame:frame]) {
        self.itemsArray = titles;
        self.buttonHeight = frame.size.height;
        self.buttonWidth = (self.itemsArray.count==0?0.f:frame.size.width/self.itemsArray.count);
        self.delegate = theDelegate;
        
        [self configView];
        return self;
    }
    return nil;
}

- (instancetype)initWithTitles:(NSArray *)titles frame:(CGRect)frame callback:(callbackBlock)block{
    if (self = [super initWithFrame:frame]) {
        self.itemsArray = titles;
        self.buttonHeight = frame.size.height;
        self.buttonWidth = (self.itemsArray.count==0?0.f:frame.size.width/self.itemsArray.count);
        self.block = block;
        
        [self configView];
        return self;
    }
    return nil;
}

/*
 可以看到，两个入口方法的区别仅仅在于：一个将传进来的控制器设为代理，一个将传进来的块赋给实例变量；
 */

#pragma mark - Private

- (void)configView{
    [self defaultConfig];
    [self configSelectedBottomLine];
    [self configButton];
    [self refreshButton];
}

-(void)defaultConfig{
    self.selectedIndex = kDefauleSelectedIndex;
    self.backgroundColor = [UIColor grayColor];
    self.buttonsArray = [[NSMutableArray alloc]init];
}

-(void)configSelectedBottomLine{
    CGRect selectedLineFrame = CGRectMake(0, self.buttonHeight-kSelectedBottomLineWidth, self.buttonWidth, kSelectedBottomLineWidth);
    self.selectedBottomLine = [[UIView alloc]initWithFrame:selectedLineFrame];
    self.selectedBottomLine.backgroundColor = kSelectedBottomLineColor;
}

-(void)configButton{
    for(int n=0; n<self.itemsArray.count; n++){
        NSString *btnName = self.itemsArray[n];
        UIButton *btn = [[UIButton alloc]initWithFrame:CGRectMake(n*self.buttonWidth+0.5, 0, self.buttonWidth - 0.5, self.buttonHeight)];
        [btn setTitle:btnName forState:UIControlStateNormal];
        btn.titleLabel.font = [UIFont systemFontOfSize:kButtonTextFont];
        btn.tag = n;
        [btn addTarget:self action:@selector(buttonClick:) forControlEvents:UIControlEventTouchUpInside];
        [self.buttonsArray addObject:btn];
        [self addSubview:btn];
    }
}

-(void)refreshButton{
    for(UIButton *btn in self.buttonsArray){
        [btn setTitleColor:kButtonDefaultTitleColor forState:UIControlStateNormal];
        btn.backgroundColor = kButtonDefaultBgColor;
    }
    UIButton *selectedBtn = self.buttonsArray[self.selectedIndex];
    [selectedBtn setTitleColor:kButtonSelectedTitleColor forState:UIControlStateNormal];
    selectedBtn.backgroundColor = kButtonSelectedBgColor;
    [selectedBtn addSubview:self.selectedBottomLine];
}

//两种模式共用一个点击触发方法，在方法内判断模式类型
-(void)buttonClick:(UIButton *)button{
    self.selectedIndex = button.tag;
    [self refreshButton];
    
    if ([self.delegate respondsToSelector:@selector(segmentControl:didSelectedIndex:)]) {
        [self.delegate segmentControl:self didSelectedIndex:self.selectedIndex];
        
    }else if (self.block) {
        self.block(self.selectedIndex);
    }
}

@end
